from enum import Enum


class ConnectorTypes(Enum):
    MSSQL = 1
    ORACLE = 2
    POSTGRESQL = 3
    EXCEL = 4
    CSV = 5
    Kafka = 6
    MYSQL = 7
    Impala = 8
    Soap = 9
    CLICKHOUSE = 10
    SqLite = 11
