EVENT_LOG = 0
EVENT_EXECUTION_INITIALIZED = 1
EVENT_EXECUTION_STARTED = 2
EVENT_EXECUTION_FINISHED = 3
EVENT_EXECUTION_INTEGRATION_INITIALIZED = 4
EVENT_EXECUTION_INTEGRATION_STARTED = 5
EVENT_EXECUTION_INTEGRATION_GET_SOURCE_DATA_COUNT = 6
EVENT_EXECUTION_INTEGRATION_EXECUTE_TRUNCATE = 7
EVENT_EXECUTION_INTEGRATION_EXECUTE_TARGET = 8
EVENT_EXECUTION_INTEGRATION_EXECUTE_SOURCE = 9
EVENT_EXECUTION_INTEGRATION_FINISHED = 10
