from datetime import datetime

from ...data.domain import EntityBase


class LogData(EntityBase):
    def __init__(self,
                 TypeId: int = None,
                 Content: str = None,
                 LogDatetime: datetime = None,
                 JobId: int = None,
                 *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.TypeId = TypeId
        self.Content = Content
        self.LogDatetime = LogDatetime
        self.JobId = JobId
